package com.css.simulation.resource.scene.service;

import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.constants.DictConstants;
import api.common.pojo.constants.LogConstants;
import api.common.pojo.param.MinioParameter;
import api.common.pojo.param.scene.SceneImportParam;
import api.common.pojo.param.scene.StandardsRegulationsParam;
import api.common.pojo.po.scene.StandardsRegulationsPO;
import api.common.pojo.po.system.SceneImportPO;
import api.common.util.ObjectUtil;
import api.common.util.StringUtil;
import api.common.util.TimeUtil;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.feign.FileDownService;
import com.css.simulation.resource.log.service.LogService;
import com.css.simulation.resource.scene.mapper.StandardsRegulationsMapper;
import com.css.simulation.resource.system.service.DictService;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import feign.Response;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class StandardsRegulationsService {

    @Resource
    private StandardsRegulationsMapper standardsRegulationsMapper;

    @Resource
    private FileDownService fileDownService;
    @Autowired
    LogService logService;

    @Autowired
    DictService dictService;

    /**
     * 获取标准法规场景数据：
     */
    @SneakyThrows
    public ResponseBodyVO<String> importBzFg(StandardsRegulationsParam param) {

        MinioParameter MI = new MinioParameter();
        MI.setObjectName(param.getJsonAddress());
        Response download = fileDownService.download(MI);
        String json = download.body().toString();
        ObjectMapper objMap = new ObjectMapper();
        JsonNode root = objMap.readTree(json);
        String sceneName = root.path("场景名称").asText();
        StandardsRegulationsPO standardsRegulationsPO = new StandardsRegulationsPO();
        standardsRegulationsPO.setShare("0");
        standardsRegulationsPO.setSceneName(sceneName);
        StandardsRegulationsPO PoByName = standardsRegulationsMapper.queryStandardsRegulationsByName(standardsRegulationsPO);


        standardsRegulationsPO = standardsRegulationsPO.builder()
                .regulationsId(StringUtil.getRandomUUID())
                .sceneName(sceneName)
                .regulationType(root.path("法规类型").asText())
                .standardType(root.path("标准类型").asText())
                .maxTime(root.path("max_time").asText())
                .isDeleted("0")
                .build();
        if (null != param.getJsonAddress() && !param.getJsonAddress().equals("")) {
            standardsRegulationsPO.setJsonAddress(param.getJsonAddress());
        }
        if (null != param.getXmlAddress() && !param.getXmlAddress().equals("")) {
            standardsRegulationsPO.setXmlAddress(param.getXmlAddress());
        }
        if (null != param.getXodrAddress() && !param.getXodrAddress().equals("")) {
            standardsRegulationsPO.setXodrAddress(param.getXodrAddress());
        }
        if (null != param.getOsgbAddress() && !param.getOsgbAddress().equals("")) {
            standardsRegulationsPO.setOsgbAddress(param.getOsgbAddress());
        }
        if (null != param.getVideoAddress() && !param.getVideoAddress().equals("")) {
            standardsRegulationsPO.setVideoAddress(param.getVideoAddress());
        }
        if (null != param.getVideoPreview() && !param.getVideoPreview().equals("")) {
            standardsRegulationsPO.setVideoPreview(param.getVideoPreview());
        }

        if (ObjectUtil.isNull(standardsRegulationsPO)) {
            // -------------------------------- 保存数据到 mysql --------------------------------
            standardsRegulationsPO.setCreateTime(TimeUtil.getNowForMysql());
            standardsRegulationsPO.setModifyTime(TimeUtil.getNowForMysql());
            standardsRegulationsPO.setCreateUserId(AuthUtil.getCurrentUserId());
            standardsRegulationsMapper.saveStandardsRegulations(standardsRegulationsPO);

            // return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE, "该场景名称" + sceneName + "已存在，请检查");
        } else {
            // -------------------------------- 修改数据到 mysql --------------------------------
            standardsRegulationsPO.setRegulationsId(standardsRegulationsPO.getRegulationsId());
            standardsRegulationsPO.setModifyTime(TimeUtil.getNowForMysql());
            standardsRegulationsPO.setModifyUserId(AuthUtil.getCurrentUserId());
            standardsRegulationsMapper.updateStandardsRegulations(standardsRegulationsPO);
        }
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
    }

    //查看列表
    public List<StandardsRegulationsPO> queryStandardsRegulationsList(StandardsRegulationsParam params) {

        List<StandardsRegulationsPO> list = new ArrayList<>();

        //添加权限
        String roleCode = AuthUtil.getCurrentUserRoleCode();
        log.info("标准法规------" +roleCode);
        String userId = AuthUtil.getCurrentUserId();
        if (DictConstants.ROLE_CODE_ADMIN.equals(roleCode) || DictConstants.ROLE_CODE_SYSADMIN.equals(roleCode)) { //管理员账户
            list = standardsRegulationsMapper.queryStandardsRegulationsList(params);
            log.info("标准法规---list---" +list.size()+"----------"+params);
        } else if (DictConstants.ROLE_CODE_SUBUESR.equals(roleCode)) {
            //查询父账户
            userId = AuthUtil.getCreateUserId();
            params.setUserId(userId);
            if(params.getShare().equals("0")){
                list = standardsRegulationsMapper.queryStandardsRegulationsList(params);
            }else {
                list = standardsRegulationsMapper.queryStandardsRegulationsListByQx(params);
            }
        } else if (DictConstants.ROLE_CODE_UESR.equals(roleCode)) {
            //查询普通账户
            params.setUserId(userId);
            if(params.getShare().equals("0")){
                list = standardsRegulationsMapper.queryStandardsRegulationsList(params);
            }else {
                list = standardsRegulationsMapper.queryStandardsRegulationsListByQx(params);
            }        }
       /* if(params.getLabel()!=null&&params.getLabel().size()>0){
            if(params.getRegulationType()!=null&&params.getRegulationType().size()>0){
                List<String> regulationType= params.getRegulationType();
                regulationType.addAll(params.getLabel());
                params.setRegulationType(regulationType);
            }else{
                List<String> regulationType=new ArrayList<>();
                regulationType.addAll(params.getLabel());
                params.setRegulationType(regulationType);
            }
        }*/

        //获取权限并赋值
      /*  List<String> listType=  systemUserSceneService.querySystemUserSceneByType(DictConstants.SCENE_STANDARD);
        if(listType!=null&&listType.size()>0){
            params.setAllIds(listType);
        }else{
            List<String> listA=new ArrayList<>();
            listA.add("没有权限");
            params.setAllIds(listA);
        }*/
        return list;
    }

    public void deleteStandardsRegulations(StandardsRegulationsPO params) {
        params.setIsDeleted("1");
        params.setModifyTime(TimeUtil.getNowForMysql());
        params.setModifyUserId(AuthUtil.getCurrentUserId());
        standardsRegulationsMapper.deleteStandardsRegulations(params);

        Map map = new HashMap();
        map.put("sceneId", params.getRegulationsId());
        map.put("sceneName", params.getSceneName());
        map.put("type", DictConstants.SCENE_STANDARD);
        logService.logSceneDelete(LogConstants.SYS_LOG_SCENE_DELETE, map);
        // systemUserSceneService.deleteSystemUserSceneBySceneId(params.getRegulationsId());
    }


    public ResponseBodyVO<List<String>> queryRegulationType() {
        //添加权限
        String roleCode = AuthUtil.getCurrentUserRoleCode();
        String userId = AuthUtil.getCurrentUserId();
        if (DictConstants.ROLE_CODE_ADMIN.equals(roleCode) || DictConstants.ROLE_CODE_SYSADMIN.equals(roleCode)) { //管理员账户
            userId = "";
        } else if (DictConstants.ROLE_CODE_SUBUESR.equals(roleCode)) {
            //查询父账户
            userId = AuthUtil.getCreateUserId();
        }
        List<String> list = standardsRegulationsMapper.queryRegulationType(userId);

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS, list);

    }

    /**
     * 获取标准法规场景数据：
     */
    public SceneImportPO importMinio(SceneImportParam parms) {
        SceneImportPO sceneImportPO = new SceneImportPO();
        sceneImportPO.setId(parms.getId());
        try {
            MinioParameter MI = new MinioParameter();
            MI.setObjectName(parms.getMinioDirectory());
            List<String> list = fileDownService.listDeepOne(MI).getInfo();
            String OsgbPath = null;
            String XodrPath = null;
            int successNum = 0;
            int falseNum = 0;
            String errorMessage = "";
            for (String filePath : list) {
                if (filePath.contains("/OSGB")) {
                    OsgbPath = filePath;
                } else if (filePath.contains("/XODR")) {
                    XodrPath = filePath;
                }
            }
            for (String filePath : list) {
                if (filePath.contains("/Scenarios")) {
                    MI.setObjectName(filePath);
                    List<String> listScene = fileDownService.listDeepOne(MI).getInfo();
                    for (String scenePath : listScene) {
                        try {
                            MI.setObjectName(scenePath);
                            List<String> listAderss = fileDownService.list(MI).getInfo();
                            StandardsRegulationsPO standardsRegulationsPO = new StandardsRegulationsPO();
                            String isupdate = null;
                            for (String adress : listAderss) {
                                if (adress.contains(".json")) {
                                    MI.setObjectName(adress);
                                    Response download = fileDownService.download(MI);
                                    // -------------------------------- label.json --------------------------------
                                    String json = download.body().toString();
                                    ObjectMapper objMap = new ObjectMapper();
                                    JsonNode root = objMap.readTree(json);
                                    //判断场景名称唯一
                                    String sceneName = root.path("场景名称").asText();
                                    String regulationType = root.path("法规类型").asText();
                                    String standardType = root.path("标准类型").asText();
                                    String osgb = root.path("osgb").asText();
                                    String xodr = root.path("xodr").asText();
                                    if (StringUtil.isEmpty(sceneName)
                                            || StringUtil.isEmpty(regulationType)
                                            || StringUtil.isEmpty(standardType)
                                            || StringUtil.isEmpty(osgb)
                                            || StringUtil.isEmpty(xodr)
                                    ) {
                                        sceneImportPO.setStatus(DictConstants.SCENE_IMPORT_STATUS_2);
                                        sceneImportPO.setErrorMessage("文件" + adress + " 与标准法规场景类型不匹配，解析失败。");
                                        return sceneImportPO;
                                    }
                                    String osgbAdress = OsgbPath + "/" + osgb;
                                    String xodrAdress = XodrPath + "/" + xodr;
                                    standardsRegulationsPO.setShare("0");
                                    standardsRegulationsPO.setSceneName(sceneName);
                                    StandardsRegulationsPO PoByName = standardsRegulationsMapper.queryStandardsRegulationsByName(standardsRegulationsPO);
                                    standardsRegulationsPO.setRegulationsId(StringUtil.getRandomUUID());
                                    standardsRegulationsPO.setRegulationType(regulationType);
                                    standardsRegulationsPO.setStandardType(standardType);
                                    standardsRegulationsPO.setMaxTime(root.path("max_time").asText());
                                    standardsRegulationsPO.setIsDeleted("0");

                                    if (ObjectUtil.isNull(standardsRegulationsPO.getOsgbAddress())) {
                                        standardsRegulationsPO.setOsgbAddress(osgbAdress);
                                    }
                                    if (ObjectUtil.isNull(standardsRegulationsPO.getXodrAddress())) {
                                        standardsRegulationsPO.setXodrAddress(xodrAdress);
                                    }
                                    if (ObjectUtil.isNull(PoByName)) {
                                        isupdate = "0";
                                    } else {
                                        isupdate = "1";
                                        standardsRegulationsPO.setRegulationsId(PoByName.getRegulationsId());
                                    }
                                }
                                if (adress.contains(".json")) {
                                    standardsRegulationsPO.setJsonAddress(adress);
                                } else if (adress.contains(".xml") || adress.contains(".xosc")) {
                                    standardsRegulationsPO.setXmlAddress(adress);
                                } else if (adress.contains(".xodr")) {
                                    standardsRegulationsPO.setXodrAddress(adress);
                                } else if (adress.contains(".osgb")) {
                                    standardsRegulationsPO.setOsgbAddress(adress);
                                } else if (adress.contains(".mp4")) {
                                    standardsRegulationsPO.setVideoAddress(adress);
                                }
                            }
                            if (isupdate.equals("0")) {
                                // -------------------------------- 保存数据到 mysql --------------------------------
                                log.info("StandardsRegulationsService--importMinio 添加标准法规数据：" + standardsRegulationsPO);
                                standardsRegulationsPO.setCreateTime(TimeUtil.getNowForMysql());
                                standardsRegulationsPO.setCreateUserId(AuthUtil.getCurrentUserId());
                                standardsRegulationsPO.setModifyTime(TimeUtil.getNowForMysql());
                                standardsRegulationsMapper.saveStandardsRegulations(standardsRegulationsPO);
                            } else if (isupdate.equals("1")) {
                                // -------------------------------- 修改数据到 mysql --------------------------------
                                log.info("StandardsRegulationsService--importMinio 修改标准法规数据：" + standardsRegulationsPO);
                                standardsRegulationsPO.setModifyTime(TimeUtil.getNowForMysql());
                                standardsRegulationsPO.setModifyUserId(AuthUtil.getCurrentUserId());
                                standardsRegulationsMapper.updateStandardsRegulations(standardsRegulationsPO);
                            }
                            successNum = successNum + 1;
                        } catch (Exception e) {
                            if (ObjectUtil.isNull(errorMessage)) {
                                errorMessage = scenePath;
                            } else {
                                errorMessage = errorMessage + ";" + scenePath;
                            }
                            falseNum = falseNum + 1;
                        }
                    }
                }
            }
            sceneImportPO.setStatus(DictConstants.SCENE_IMPORT_STATUS_4);
            sceneImportPO.setSuccessNum(successNum);
            sceneImportPO.setFalseNum(falseNum);
            sceneImportPO.setErrorMessage(errorMessage);
            return sceneImportPO;
        } catch (Exception e) {
            sceneImportPO.setStatus(DictConstants.SCENE_IMPORT_STATUS_2);
            sceneImportPO.setErrorMessage("该文件地址：" + parms.getMinioDirectory() + "解析失败，请检查");
            return sceneImportPO;
        }
    }

    /**
     * 获取标准法规场景数据：
     */
    public void instStandardsRegulation(StandardsRegulationsPO po) {
        StandardsRegulationsPO syPo = standardsRegulationsMapper.queryStandardsRegulationsById(po.getRegulationsId());
        syPo.setShare("1");
        StandardsRegulationsPO PoByName = standardsRegulationsMapper.queryStandardsRegulationsByName(syPo);

        if (ObjectUtil.isNull(PoByName)) {
            // -------------------------------- 保存数据到 mysql --------------------------------
            syPo.setCreateTime(TimeUtil.getNowForMysql());
            syPo.setCreateUserId(AuthUtil.getCurrentUserId());
            syPo.setModifyTime(TimeUtil.getNowForMysql());
            syPo.setModifyUserId("");
            syPo.setRegulationsId(api.common.util.StringUtil.getRandomUUID());
            standardsRegulationsMapper.saveStandardsRegulations(syPo);
        } else {
            // -------------------------------- 修改数据到 mysql --------------------------------
            syPo.setModifyTime(TimeUtil.getNowForMysql());
            syPo.setModifyUserId(AuthUtil.getCurrentUserId());
            syPo.setRegulationsId(PoByName.getRegulationsId());
            standardsRegulationsMapper.updateStandardsRegulations(syPo);
        }

    }
}
